==> pmml-sklearn-1.7.37-sources.jar: sklearn/pipeline/FeatureUnion.java <==
/*
 * Copyright (c) 2017 Villu Ruusmann
 *
 * This file is part of JPMML-SkLearn
 *
 * JPMML-SkLearn is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JPMML-SkLearn is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with JPMML-SkLearn.  If not, see <http://www.gnu.org/licenses/>.
 */
package sklearn.pipeline;

import java.util.ArrayList;
import java.util.List;

import org.dmg.pmml.DataType;
import org.dmg.pmml.OpType;
import org.jpmml.converter.Feature;
import org.jpmml.python.TupleUtil;
import org.jpmml.sklearn.SkLearnEncoder;
import sklearn.HasHead;
import sklearn.SkLearnTransformer;
import sklearn.Transformer;
import sklearn.TransformerUtil;

public class FeatureUnion extends SkLearnTransformer implements HasHead {

	public FeatureUnion(String module, String name){
		super(module, name);
	}

	@Override
	public OpType getOpType(){
		throw new UnsupportedOperationException();
	}

	@Override
	public DataType getDataType(){
		throw new UnsupportedOperationException();
	}

	@Override
	public List<Feature> encodeFeatures(List<Feature> features, SkLearnEncoder encoder){
		List<? extends Transformer> transformers = getTransformers();

		List<Feature> result = new ArrayList<>();

		for(Transformer transformer : transformers){
			List<Feature> transformerFeatures = new ArrayList<>(features);

			transformerFeatures = transformer.encode(transformerFeatures, encoder);

			result.addAll(transformerFeatures);
		}

		return result;
	}

	@Override
	public Transformer getHead(){
		List<? extends Transformer> transformers = getTransformers();

		if(!transformers.isEmpty()){
			Transformer transformer = transformers.get(0);

			return TransformerUtil.getHead(transformer);
		}

		return null;
	}

	public List<? extends Transformer> getTransformers(){
		List<Object[]> transformerList = getTransformerList();

		return TupleUtil.extractElementList(transformerList, 1, Transformer.class);
	}

	public List<Object[]> getTransformerList(){
		return getTupleList("transformer_list");
	}
}

==> pmml-sklearn-1.7.51-sources.jar: sklearn/pipeline/FeatureUnion.java <==
/*
 * Copyright (c) 2017 Villu Ruusmann
 *
 * This file is part of JPMML-SkLearn
 *
 * JPMML-SkLearn is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JPMML-SkLearn is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with JPMML-SkLearn.  If not, see <http://www.gnu.org/licenses/>.
 */
package sklearn.pipeline;

import java.util.ArrayList;
import java.util.List;

import org.dmg.pmml.DataType;
import org.dmg.pmml.OpType;
import org.jpmml.converter.Feature;
import org.jpmml.python.TupleUtil;
import org.jpmml.sklearn.SkLearnEncoder;
import sklearn.HasHead;
import sklearn.SkLearnTransformer;
import sklearn.Step;
import sklearn.StepUtil;
import sklearn.Transformer;

public class FeatureUnion extends SkLearnTransformer implements HasHead {

	public FeatureUnion(String module, String name){
		super(module, name);
	}

	@Override
	public OpType getOpType(){
		throw new UnsupportedOperationException();
	}

	@Override
	public DataType getDataType(){
		throw new UnsupportedOperationException();
	}

	@Override
	public List<Feature> encodeFeatures(List<Feature> features, SkLearnEncoder encoder){
		List<? extends Transformer> transformers = getTransformers();

		List<Feature> result = new ArrayList<>();

		for(Transformer transformer : transformers){
			List<Feature> transformerFeatures = new ArrayList<>(features);

			transformerFeatures = transformer.encode(transformerFeatures, encoder);

			result.addAll(transformerFeatures);
		}

		return result;
	}

	@Override
	public Step getHead(){
		List<? extends Transformer> transformers = getTransformers();

		if(!transformers.isEmpty()){
			Transformer transformer = transformers.get(0);

			return StepUtil.getHead(transformer);
		}

		throw new IllegalArgumentException();
	}

	public List<? extends Transformer> getTransformers(){
		List<Object[]> transformerList = getTransformerList();

		return TupleUtil.extractElementList(transformerList, 1, Transformer.class);
	}

	public List<Object[]> getTransformerList(){
		return getTupleList("transformer_list");
	}
}
